; Save/Load Feature by yoshifanatic
; This patch adds a save/load feature to Jurassic Park, fixing one of its bigger flaws.
; The game will autosave when you start a new game, after respawning after death, and whenever a checkpoint is triggered, which means it will occur when you touch a pole, when entering a building, or when you appear outside after leaving a building.
; In addition, the original title screen menu will be replaced with a new one to allow selecting a file.
; As a bonus, the new title screen menu will also contain a song test. When the cursor is hovering over that option, press A/B to increment the song ID, X/Y to decrement it, and select to play that music.
; Apply this to V1.0 USA Jurassic Park ROM.
; Here is a list of what is saved:
; - Objective clear flags, except the one for boarding the helicopter, since that one wouldn't make sense.
; - Current checkpoint X/Y/Z position
; - Collected ID cards
; - Collected batteries
; - Security level
; - Motion Sensors Active flag
; - Current primary and secondary weapon and their ammo
; - Current life count
; - How many eggs are remaining
; - The latest message you got from touching a pole.
; - Damage taken
; - Which gates are open/closed
; - The flags that indicate what dinosaurs you killed in the interior areas
; - The state of every exterior sprite
; - High score data
; - The music and sound setting on the title screen

!SRAM = $700000
!CursorOverMenuOptionIndex = !SRAM						; What line of text the cursor is hoving over.
!MusicSetting = !SRAM+$02							; Music setting on the title screen
!SoundSetting = !SRAM+$04							; Sound setting on the title screen
!SongTestSetting = !SRAM+$06							; The song that will play when selecting the song test menu
!CurrentSaveFile = !SRAM+$08							; The current save file to index
!SaveInProgressFlag = !SRAM+$0A							; Used to prevent the game from crashing while saving, due to the save routine taking long enough that it causes IRQs to stack.
!HighScoreChecksum = !SRAM+$0C							; Checksum of the high score data
!HighScoreData = !HighScoreChecksum+$02						; The High score and name data
!SaveFile1Start = !SRAM+$8C							; Save file 1's data
!SaveFile2Start = !SaveFile1Start+(!SaveFileSize*$01)+$02			; Save file 2's data
!SaveFile3Start = ((!SaveFile1Start+(!SaveFileSize*$02)+$04)&$FF7FFF)+$010000	; Save file 3's data

!FileSelectTextInitialXDisp = $20						; How far the file select text is from the left edge of the screen
!FileSelectTextInitialYDisp = $18						; How far the file select text is from the top edge of the screen
!FileSelectTextYDisp = $16							; The distance between each line of file select text
!NumberOfEggs = $12								; How many eggs are in the game. Only change this if you add/remove eggs from the game.
!SaveFileSize = $5EA6
!ChecksumCompliment = $BEEF

org $8080D5						; Check if save files are valid on startup.
	JSL.l VerifySaveFiles

org $80A3C3
	JSL.l SaveGameWhenRespawning
	NOP #2

org $80A83B
	JSL.l SaveGameWhenEnteringBuilding
	NOP #2

org $80FFD8						; Set ROM contents to be ROM/RAM/SRAM
	db $02

org $80FFD8						; Set SRAM size to 128 KB
	db $07

org $8180B6
	JSL.l SaveGameAfterLeavingBuilding
	NOP #2

org $81D636
	REP.b #$30
	LDX.w #$8CE768
	LDA.w #$8CE768>>16
org $81D645
	JSL.l HandleNewTitleScreenFiles
	BRA.b CODE_81D66F

org $81D66F
CODE_81D66F:

org $81D6DF						; Disable menu option highlighting HDMA on title screen.
	NOP #2

org $81D77C						; No sense in checking for whether to fade to the high score screen here.
	JMP.w $81D7A3
	NOP #2
CODE_81D781:

org $81D7A3						; Add File select to title screen
	JSL.l FileSelectScreen

org $81D819						; Save game upon starting new file.
	JML.l SaveOrLoadGameAtGameStart

org $81D87A						; Disable start from starting a file when the cursor is not over a file.
	db $80

org $81D87F						; Change what happens when the JP box is "opened"
	LDA.w $044D
	BNE.b CODE_81D8B8
	LDA.w $0AF1
	AND.w #$E0C0
	BEQ.b +
	JMP.w $81D90A

+:
	RTS

org $81D8B8
CODE_81D8B8:

org $81D8BB
	JML.l HandleCursorCollision
	NOP

org $81D916						; Change how $044D is handled, so title screen state 02 is the one that starts the game.
	NOP #3
	INC.w $044D

org $81D93D						; Selecting "HIGH SCORES" uses title screen state 03 now.
	LDA.w #$0004

org $81D95B
	JSL.l SaveMusicSetting
	NOP #2

org $81D9A4
	JSL.l SaveSoundSetting

org $81D9B8
	JSL.l SaveSoundSetting

org $81D9CC
	JSL.l SaveSoundSetting

org $81DB43
	JML.l JPIconHijack

org $81E397						; Save High Scores when a new one is registered
	JSL.l SaveNewHighScore

org $83DDDA
	JSL.l SaveGameAtPole
	NOP #2

org $8F8000
CursorGFX:
	incbin "FixedCursor.bin"
.End:

assert !FileSelectTextYDisp >= $10,"\!FileSelectTextYDisp's value must be greater than $10!"

!CurrentFileChecksumRAM = $01
!CurrentScoreChecksumRAM = $01

org $83F4F2
VerifySaveFiles:
	LDA.w #$0000
	STA.l !SaveInProgressFlag
	STA.l !CursorOverMenuOptionIndex
	STA.l !SongTestSetting
	LDA.w #$0002
	STA.l !CurrentSaveFile
--
	JSR.w GetSRAMPointer
	LDA.w #$0000
	PHA
+:
	LDX.w #!SaveFileSize
-
	JSR.w ReadSRAMByte
	CLC
	ADC.b !CurrentFileChecksumRAM,s
	STA.b !CurrentFileChecksumRAM,s
	DEX
	DEX
	BNE.b -
	SEC
	SBC.w #!ChecksumCompliment
	CMP.b [$FC]
	BNE.b .InvalidSaveFile
.NextFile:
	PLA
	LDA.l !CurrentSaveFile
	BEQ.b +
	DEC
	STA.l !CurrentSaveFile
	JMP.w --

+
	LDA.w #$0000
	PHA
	LDY.w #$007C
	TYX
-
	LDA.l !HighScoreData,x
	CLC
	ADC.b !CurrentScoreChecksumRAM,s
	STA.b !CurrentScoreChecksumRAM,s
	DEX
	DEX
	DEY
	DEY
	BPL.b -
	SEC
	SBC.w #!ChecksumCompliment
	CMP.l !HighScoreChecksum
	BEQ.b .ValidHighScores
	LDA.w #$0000
	STA.l !MusicSetting
	LDA.w #$0002
	STA.l !SoundSetting
	JSL.l $81DF23				; Initialize High Scores routine.
	BRA.b .InvalidHighScores

.ValidHighScores:
	JSL.l LoadSavedHighScoreData
.InvalidHighScores:
	PLA
	JSL.l SaveHighScores
	RTL

.InvalidSaveFile:
	LDA.l !CurrentSaveFile
	JSR.w GetSRAMPointer
	LDA.w #$0000
	LDX.w #!SaveFileSize
-
	JSR.w StoreSRAMByte
	DEX
	DEX
	BNE.b -
	LDA.w #$0000
	STA.b [$FC]
	JMP.w .NextFile

SaveGameWhenRespawning:
	LDA.w #$03C0
	STA.w $0E3D
	LDA.w $0385			;\ If in demo mode, don't save the game.
	BNE.b +				;/
	JSL.l SaveGame
+:
	RTL

SaveGameWhenEnteringBuilding:
	LDA.w $0E46
	BNE.b +
	LDA.w $0DFB
	STA.w $0E35			;\  Save checkpoint position here, otherwise it won't get saved.
	LDA.w $0DFD			;|
	STA.w $0E37			;|
	LDA.w $7E72BE			;|
	STA.w $0E33			;/
	JSL.l SaveGame
+:
	LDA.w $0DFB
	STA.w $0E46
	RTL

SaveGameAfterLeavingBuilding:
	LDA.w #$0004
	STA.w $0B21
	BRA.b SaveGame

SaveGameAtPole:
	JSL.l SaveGame
	STZ.w $0DFF
	LDX.w $037F
	RTL

SaveOrLoadGameAtGameStart:
	PHP
	REP.b #$30
	LDA.l !CurrentSaveFile
	JSR.w GetSRAMPointer
	JSR.w ReadSRAMByte
	JSR.w ORASRAMByte
	JSR.w ORASRAMByte
	JSR.w ORASRAMByte
	CMP.w #$0000
	BEQ.b .NewFile
	JSL.l LoadGame
	BRA.b +

.NewFile
	JSL.l SaveGame
+:
	PLP
	JML.l $818105

SaveGame:
	LDA.l !SaveInProgressFlag
	BEQ.b +
	RTL

+:
	PHB
	PEA.w $7E7E
	PLB
	PLB
	PHP
	REP.b #$30
	LDA.w #$0000
	PHA
	LDA.w #$FFFF
	STA.l !SaveInProgressFlag
	LDA.l !CurrentSaveFile
	JSR.w GetSRAMPointer
	LDX.w #(SingleRAMAddressesToSave_End-SingleRAMAddressesToSave)-$02
-
	PHY
	LDA.l SingleRAMAddressesToSave,x
	TAY
	LDA.w $0000,y
	PLY
	JSR.w StoreSRAMByteAndStoreTempChecksum
	DEX
	DEX
	BPL -
	LDX.w #$0014
-
	LDA.w $0289,x			; Motion sensor active flag, all eggs collected flag, and primary/secondary weapon data
	JSR.w StoreSRAMByteAndStoreTempChecksum
	DEX
	DEX
	BPL -
	LDX.w #$0340
-
	LDA.w $1AA4,x			; Interior Enemy Load Status Flags
	JSR.w StoreSRAMByteAndStoreTempChecksum
	DEX
	DEX
	BPL -
	LDX.w #$0018
-
	LDA.w $1E14,x			; Killed All Dinosaurs on Floor table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	DEX
	DEX
	BPL -
	LDX.w #$035C
-
	LDA.w $7E3038,x			; Exterior sprite IDs ($7E3038-$7E3395)
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E2090,x			; Exterior sprite X position
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E2864,x			; Exterior sprite Y position
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E3422,x			; Exterior sprite Animation frame
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E43CA,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E3BF6,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E3FE0,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E380C,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E6ED8,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E5372,x			; Exterior sprite movement direction
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E247A,x			; Exterior sprite sub X Pos
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E2C4E,x			; Exterior sprite sub Y Pos
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E7E80,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E4B9E,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E6AEE,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E8654,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E47B4,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E4F88,x			; Exterior sprite Z position
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E5B46,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E5F30,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E631A,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E6704,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E7A96,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E72C2,x			; Exterior sprite damage table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E826A,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E76AC,x			; Unknown Exterior sprite table
	JSR.w StoreSRAMByteAndStoreTempChecksum
	LDA.w $7E575C,x			; Exterior sprite speed
	JSR.w StoreSRAMByteAndStoreTempChecksum
	DEX
	DEX
	BMI +
	JMP.w -

+:
	SEC
	SBC.w #!ChecksumCompliment
	STA.b [$FC]
	LDA.w #$0000
	STA.l !SaveInProgressFlag
	PLA
	PLP
	PLB
	RTL

LoadGame:
	;PHB
	;PEA.w $7E7E
	;PLB
	;PLB
	PHP
	REP.b #$30
	LDA.l !CurrentSaveFile
	JSR.w GetSRAMPointer
	LDX.w #(SingleRAMAddressesToSave_End-SingleRAMAddressesToSave)-$02
-
	JSR.w ReadSRAMByte
	PHY
	PHA
	LDA.l SingleRAMAddressesToSave,x
	TAY
	PLA
	STA.w $0000,y
	PLY
	DEX
	DEX
	BPL -
	LDX.w #$0014
-
	JSR.w ReadSRAMByte
	STA.w $0289,x			; All eggs collected flag and primary/secondary weapon data
	DEX
	DEX
	BPL -
	LDX.w #$0340
-
	JSR.w ReadSRAMByte
	STA.w $1AA4,x			; Interior Enemy Load Status Flags
	DEX
	DEX
	BPL -
	LDX.w #$0018
-
	JSR.w ReadSRAMByte
	STA.w $1E14,x			; Killed All Dinosaurs on Floor table
	DEX
	DEX
	BPL -
	LDX.w #$035C
-
	JSR.w ReadSRAMByte
	STA.w $7E3038,x			; Exterior sprite IDs ($7E3038-$7E3395)
	JSR.w ReadSRAMByte
	STA.w $7E2090,x			; Exterior sprite X position
	JSR.w ReadSRAMByte
	STA.w $7E2864,x			; Exterior sprite Y position
	JSR.w ReadSRAMByte
	STA.w $7E3422,x			; Exterior sprite Animation frame
	JSR.w ReadSRAMByte
	STA.w $7E43CA,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E3BF6,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E3FE0,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E380C,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E6ED8,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E5372,x			; Exterior sprite movement direction
	JSR.w ReadSRAMByte
	STA.w $7E247A,x			; Exterior sprite sub X position
	JSR.w ReadSRAMByte
	STA.w $7E2C4E,x			; Exterior sprite sub Y position
	JSR.w ReadSRAMByte
	STA.w $7E7E80,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E4B9E,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E6AEE,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E8654,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E47B4,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E4F88,x			; Exterior sprite Z position
	JSR.w ReadSRAMByte
	STA.w $7E5B46,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E5F30,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E631A,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E6704,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E7A96,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E72C2,x			; Exterior sprite damage table
	JSR.w ReadSRAMByte
	STA.w $7E826A,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E76AC,x			; Unknown Exterior sprite table
	JSR.w ReadSRAMByte
	STA.w $7E575C,x			; Exterior sprite speed
	DEX
	DEX
	BMI +
	JMP.w -

+:
	PLP
	;PLB
	RTL

SingleRAMAddressesToSave:
.EggRAM
	dw $0E0F			; Remaining Eggs
	dw $0253			; John Hammond ID Card Collected Flag
	dw $0255			; Ellie Sattler ID Card Collected Flag
	dw $0257			; Robert Muldoon ID Card Collected Flag
	dw $0259			; Alan Grant ID Card Collected Flag
	dw $025B			; Donald Gennaro ID Card Collected Flag
	dw $025D			; Ray Arnold ID Card Collected Flag
	dw $025F			; Dennis Nedry ID Card Collected Flag
	dw $0261			; Dr. Wu ID Card Collected Flag
	dw $0263			; Ian Malcolm ID Card Collected Flag
	dw $0265			; Have Security Level 1 Flag
	dw $0267			; Have Security Level 2 Flag
	dw $026B			; Generator Active Flag
	dw $026D			; Main System Restored Flag
	dw $026F			; Visitor Center Secured Flag
	dw $1DF5			; Ship Contacted Flag
	dw $1DF7			; Ship Free of Dinos Flag
	dw $0271			; Ship Objectives Clear Flag
	dw $0273			; Nerve Gas Deployed Flag
	dw $0275			; Mainland Contacted Flag
	;dw $0277			; Board Helicopter Flag
	dw $0287			; Motion Sensor Active Flag
.EndOfSingleByteRAM:
	dw $1DE4			; Collected Batteries 1
	dw $1DE6			; Collected Batteries 2
	dw $1DE8			; Collected Batteries 3
	dw $1DEA			; Collected Batteries 4
	dw $1DEC			; Collected Batteries 5
	dw $1DEE			; Collected Batteries 6/Nerve Gas Canister Collected (Low Byte)
.EndOfPercentCompletionRAM:
	dw $1DF0			; Nerve Gas Canister Collected (High byte)
	dw $024D			;\ Gate open flags
	dw $024E			;|
	dw $0251			;/
	dw $02A3			; Current Lives
	dw $02EB			; Damage Taken
	dw $037F			; Latest Pole Message
	dw $0E33			;\ Checkpoint X/Y/Z
	dw $0E35			;|
	dw $0E37			;/
.End:

SaveFileLocations:
	dd !SaveFile1Start
	dd !SaveFile2Start
	dd !SaveFile3Start

FileSelectScreen:
	LDA.l !MusicSetting
	STA.w $02EF
	LDA.l !SoundSetting
	BEQ.b .Mono
	CMP.w #$0002
	BEQ.b .Stereo
	LDA.w #$0002
	BRA.b +

.Stereo:
	LDA.w #$0000
	BRA.b +

.Mono:
	LDA.w #$0010
+:
	STA.w $02F1
	PHK
	PEA.w .Return1-$01
	PEA.w $81CF07-$01
	JML.l $81D98E					; Routine that applies the sound setting.
.Return1:
-:
	JSL.l $819EF1					; Wait a frame
	JSR.w ProcessCursor
	JSR.w DrawFileSelectText
	LDA.w $044D					;\ 
	CMP.w #$0004					;| If "HIGH SCORES" was selected, fade out to that screen
	BEQ.b +						;|
	CMP.w #$0002					;| If the player selected a file, start the game.
	BNE.b -						;/

	SEP.b #$20
	LDA.b #$00
	RTL

+:
	TSX
	INX
	INX
	INX
	TXS
	JML.l CODE_81D781

ProcessCursor:
	PHK
	PEA.w .Return1-$01
	PEA.w $81CF07-$01
	JML.l $81D81D			; Some other cursor related routine.
.Return1:
	PHK
	PEA.w .Return2-$01
	PEA.w $81CF07-$01
	JML.l $81D9D1			; Routine that moves the cursor?
.Return2:
	PHK
	PEA.w .Return3-$01
	PEA.w $81CF07-$01
	JML.l $81DB09			; Routine that moves the cursor
.Return3:
	RTS

!XYDisp = $03
!LetterIndex = $05

DrawFileSelectText:
	PHB
	PHK
	PLB
	LDA.w #$0000
	PHA
	LDA.w #(!FileSelectTextInitialYDisp<<8)|(!FileSelectTextInitialXDisp)
	PHA

	LDY.w #FileSelectTextPtrs_SelectFileMenu-FileSelectTextPtrs
	LDA.w $044D 
	CMP.w #$0003
	BNE.b +
	LDY.w #FileSelectTextPtrs_EraseFileMenu-FileSelectTextPtrs
+:
	JSR.w .DrawLine
	JSR.w .DrawLine
	JSR.w .DrawLine
	JSR.w .DrawLine
	JSR.w .DrawLine
	LDA.w $044D 
	CMP.w #$0003
	BNE.b +
	JMP.w .Return

+:
	LDA.w $02EF
	BEQ.b +
	INY
	INY
+
	JSR.w .DrawLine
	LDA.w $037C
	AND.w #$0003
	BEQ.b +
	INY
	INY
	DEC
	BEQ.b +
	INY
	INY
+
	JSR.w .DrawLine
	JSR.w .DrawLine
.Return:
	JSR.w HandleFilePercents
	PLA
	PLA
	PLB
	RTS

.DrawLine:
	LDA.w FileSelectLineIndex,y
	CMP.l !CursorOverMenuOptionIndex
	BNE.b .DontSkipDrawing
	LDA.w $049D
	AND.w #$0003
	BNE.b .DontSkipDrawing
	JMP.w .SkipDrawing

.DontSkipDrawing:
	LDA.w FileSelectLineIndex,y
	TAX
	LDA.w TextOAMIndex,x
	TAX
-
	PHY
	LDA.w FileSelectTextPtrs,y
	CLC
	ADC.b !LetterIndex+$02,s
	TAY
	LDA.w $0000,y
	CMP.w #$FFFF
	BEQ.b +
	PLY
	STA.w $04A1,x
	LDA.b !XYDisp,s
	STA.w $049F,x
	LDA.b !LetterIndex,s
	INC
	INC
	STA.b !LetterIndex,s
	LDA.b !XYDisp,s
	CLC
	ADC.w #$0010
	STA.b !XYDisp,s
	INX
	INX
	INX
	INX					; No need to store to the upper OAM buffer, as the game clears it every frame and that value is the needed one anyway.
	BRA.b -

+:
	PLY
.SkipDrawing:
	LDA.w #$0000
	STA.b !LetterIndex,s
	LDA.b !XYDisp,s
	AND.w #$FF00
	CLC
	ADC.w #(!FileSelectTextYDisp<<8)|(!FileSelectTextInitialXDisp)
	STA.b !XYDisp,s
	TYA
	CLC
	ADC.w FileSelectLineIndexOffset,y
	TAY
	RTS

!FilePercent = $03
!OnesDigit = $03
!TempCurrentSaveFile = $01

HandleFilePercents:
	LDA.w #$0000
	PHA
	LDA.w #$0002
	PHA
--:
	JSR.w GetSRAMPointer
	JSR.w ReadSRAMByte
	JSR.w ORASRAMByte
	JSR.w ORASRAMByte
	JSR.w ORASRAMByte
	CMP.w #$0000
	BEQ.b .NewFile
	LDA.b !TempCurrentSaveFile,s
	JSR.w GetSRAMPointer
	TYA
	CLC
	ADC.w #(SingleRAMAddressesToSave_End-SingleRAMAddressesToSave_EndOfPercentCompletionRAM)
	TAY
	CPY.w #$8000
	BCC.b +
	LDY.w #$0000
	INC.b $FA
+:
	LDX.w #(SingleRAMAddressesToSave_EndOfPercentCompletionRAM-SingleRAMAddressesToSave)-$02
-
	JSR.w ReadSRAMByte
	CPX.w #SingleRAMAddressesToSave_EggRAM-SingleRAMAddressesToSave
	BEQ.b +++
	CPX.w #SingleRAMAddressesToSave_EndOfSingleByteRAM-SingleRAMAddressesToSave
	BCC.b ++
	PHA
	AND.w #$00FF
	BEQ.b +
	ORA.w #$FF00
+:
	CLC
	ADC.b !FilePercent+$02,s
	STA.b !FilePercent+$02,s
	PLA
	XBA
	AND.w #$00FF
	BEQ.b +
	ORA.w #$FF00
+:
	BRA.b ++

+++:
	SEC
	SBC.w #!NumberOfEggs
++:
	CLC
	ADC.b !FilePercent,s
	STA.b !FilePercent,s
	DEX
	DEX
	BPL.b -
	BRA.b +

.NewFile:
	JMP.w ++++

+:
	LDA.b !TempCurrentSaveFile,s
	ASL
	TAX
	LDA.l FilePercentOAMIndex,x
	TAY
	LDA.b !FilePercent,s
	EOR.w #$FFFF
	INC
	STA.l $004204
	SEP.b #$20
	LDA.b #!NumberOfEggs+((SingleRAMAddressesToSave_EndOfSingleByteRAM-SingleRAMAddressesToSave-$02)/$02)+(SingleRAMAddressesToSave_EndOfPercentCompletionRAM-SingleRAMAddressesToSave_EndOfSingleByteRAM)
	STA.l $004206
	NOP #7
	REP.b #$21
	LDA.l $004214
	BEQ.b +
	LDA.w #$3E96
	STA.w $049F+$00,y
	LDA.w #$3E94
	STA.w $049F+$04,y
	STA.w $049F+$08,y
	LDA.w #$3ED0
	STA.w $049F+$0C,y
	BRA.b +++

+:
	LDA.w #$3EB8
	STA.w $049F+$04,y
	LDA.w #$3ED0
	STA.w $049F+$0C,y
	PHY
	LDY.w #$0000
	LDA.l $004216
	ASL					; If new things are added to the completion percent calculation, then this will cause issues.
-:
	CMP.w #$000A
	BCC.b +
	SEC
	SBC.w #$000A
	INY
	BRA.b -

+:
	STA.b !OnesDigit+$02,s
	TYA
	BEQ.b +
	ASL
	TAX
	PLY
	LDA.l NumberTiles,x
	STA.w $049F+$04,y
	PHY
+:
	PLY
	LDA.b !OnesDigit,s
	ASL
	TAX
	LDA.l NumberTiles,x
	STA.w $049F+$08,y
+++:
	LDA.w #$0000
	STA.b !FilePercent,s
++++:
	LDA.b !TempCurrentSaveFile,s
	BEQ.b +
	LDA.b !TempCurrentSaveFile,s
	DEC
	STA.b !TempCurrentSaveFile,s
	JMP.w --

+:
	LDY.w #$0000
	LDA.l !SongTestSetting
-:
	CMP.w #$000A
	BCC.b +
	SEC
	SBC.w #$000A
	INY
	BRA.b -

+:
	ASL
	TAX
	LDA.l NumberTiles,x
	PHA
	TYA
	ASL
	TAX
	LDY.w #$0182
	LDA.l NumberTiles,x
	STA.w $049F,y
	PLA
	STA.w $049F+$04,y
	PLA
	PLA
	RTS

FilePercentOAMIndex:
	dw $002A,$005A,$008A

TextOAMIndex:
	dw $0008,$0038,$0068,$0098,$00C8,$00F8,$0128,$0158
	dw $0008,$0038,$0068,$0098,$00C8,$00F8,$0128,$0158

NumberTiles:
	dw $3E94,$3E96,$3E98,$3E9A,$3E9C,$3E9E,$3EB0,$3EB2,$3EB4,$3EB6,$3EB8

FileSelectTextPtrs:
.SelectFileMenu:
	dw FileSelectText_Line1,FileSelectText_Line2,FileSelectText_Line3,FileSelectText_Line4,FileSelectText_Line5
	dw MusicSettingText_Line1,MusicSettingText_Line2
	dw SoundSettingText_Line1,SoundSettingText_Line2,SoundSettingText_Line3
	dw MusicTestText_Line1

.EraseFileMenu:
	dw FileSelectText_Line1,FileSelectText_Line2,FileSelectText_Line3,EraseFileText_Line4,EraseFileText_Line5

FileSelectLineIndex:
	dw $0000,$0002,$0004,$0006,$0008
	dw $000A,$000A
	dw $000C,$000C,$000C
	dw $000E

	dw $0010,$0012,$0014,$0016,$0018

FileSelectLineIndexOffset:
	dw $0002,$0002,$0002,$0002,$0002
	dw $0004,$0002
	dw $0006,$0004,$0002
	dw $0002

	dw $0002,$0002,$0002,$0002,$0002

FileSelectText:
.Line1:
	dw $3E3A,$3E50,$3E56,$3E38,$3EDE,$3E96,$3EB8,$3EB8,$3EB8,$3E5A,$3E38,$3E7C	; "FILE 1...NEW"
	dw $FFFF

.Line2:
	dw $3E3A,$3E50,$3E56,$3E38,$3EDE,$3E98,$3EB8,$3EB8,$3EB8,$3E5A,$3E38,$3E7C	; "FILE 2...NEW"
	dw $FFFF

.Line3:
	dw $3E3A,$3E50,$3E56,$3E38,$3EDE,$3E9A,$3EB8,$3EB8,$3EB8,$3E5A,$3E38,$3E7C	; "FILE 3...NEW"
	dw $FFFF

.Line4:
	dw $3EDE,$3E38,$3E72,$3E30,$3E74,$3E38,$3EDE,$3E3A,$3E50,$3E56,$3E38,$3EDE	; " ERASE FILE "
	dw $FFFF

.Line5:
	dw $3E3E,$3E50,$3E3C,$3E3E,$3EDE,$3EDE,$3E74,$3E34,$3E5C,$3E72,$3E38,$3E74	; "HIGH  SCORES"
	dw $FFFF

MusicSettingText:
.Line1:
	dw $3EDE,$3EDE,$3E58,$3E78,$3E74,$3E50,$3E34,$3EDE,$3E5C,$3E5A,$3EDE,$3EDE	; "  MUSIC ON  "
	dw $FFFF

.Line2:
	dw $3EDE,$3EDE,$3E58,$3E78,$3E74,$3E50,$3E34,$3EDE,$3E5C,$3E3A,$3E3A,$3EDE	; "  MUSIC OFF "
	dw $FFFF

SoundSettingText:
.Line1:
	dw $3EDE,$3EDE,$3E58,$3E5C,$3E5A,$3E5C,$3EDE,$3E74,$3E3A,$3E7E,$3EDE,$3EDE	; "  MONO SFX  "
	dw $FFFF

.Line2:
	dw $3EDE,$3E74,$3E76,$3E38,$3E72,$3E38,$3E5C,$3EDE,$3E74,$3E3A,$3E7E,$3EDE	; " STEREO SFX "
	dw $FFFF

.Line3:
	dw $3E74,$3E78,$3E72,$3E72,$3E5C,$3E78,$3E5A,$3E36,$3EDE,$3E74,$3E3A,$3E7E	; "SURROUND SFX"
	dw $FFFF

MusicTestText:
.Line1:
	dw $3E74,$3E5C,$3E5A,$3E3C,$3EDE,$3E76,$3E38,$3E74,$3E76,$3EDE,$3E94,$3E94	; "SONG TEST 00"
	dw $FFFF

EraseFileText:
.Line4:
	dw $3E3A,$3E50,$3E56,$3E38,$3EDE,$3EDE,$3E74,$3E38,$3E56,$3E38,$3E34,$3E76	; "FILE  SELECT"
	dw $FFFF

.Line5:
	dw $3E38,$3E72,$3E30,$3E74,$3E38,$3EDE,$3E74,$3E34,$3E5C,$3E72,$3E38,$3E74	; "ERASE SCORES"
	dw $FFFF

HandleCursorCollision:
	LDY.w #$05F9
	LDA.b $34
	CMP.w #!FileSelectTextInitialXDisp
	BCC.b ++
	CMP.w #!FileSelectTextInitialXDisp+$C0
	BCS.b ++
	LDX.w #$0000
+:
	LDA.b $36
-
	CMP.l .FileSelectLinePos,x
	BCC.b +
	INX
	INX
	CPX.w #.FileSelectLinePos_End-.FileSelectLinePos
	BCC.b -
++:
	LDX.w #$0000
+:
	LDA.w $044D 
	CMP.w #$0003
	BNE.b ++
	LDA.l .EraseFileOptionIndex,x
	STA.l !CursorOverMenuOptionIndex
	TXA
	CLC
	ADC.w #.EraseFileLineRoutines
	BRA.b +

++:
	LDA.l .FileSelectOptionIndex,x
	STA.l !CursorOverMenuOptionIndex
	TXA
	CLC
	ADC.w #.FileSelectLineRoutines
+:
	TAX
	JMP.w ($0000,x)

.FileSelectLinePos:
	dw !FileSelectTextInitialYDisp,!FileSelectTextInitialYDisp+$10
	dw !FileSelectTextInitialYDisp+!FileSelectTextYDisp,!FileSelectTextInitialYDisp+$10+!FileSelectTextYDisp
	dw !FileSelectTextInitialYDisp+(!FileSelectTextYDisp*$02),!FileSelectTextInitialYDisp+$10+(!FileSelectTextYDisp*$02)
	dw !FileSelectTextInitialYDisp+(!FileSelectTextYDisp*$03),!FileSelectTextInitialYDisp+$10+(!FileSelectTextYDisp*$03)
	dw !FileSelectTextInitialYDisp+(!FileSelectTextYDisp*$04),!FileSelectTextInitialYDisp+$10+(!FileSelectTextYDisp*$04)
	dw !FileSelectTextInitialYDisp+(!FileSelectTextYDisp*$05),!FileSelectTextInitialYDisp+$10+(!FileSelectTextYDisp*$05)
	dw !FileSelectTextInitialYDisp+(!FileSelectTextYDisp*$06),!FileSelectTextInitialYDisp+$10+(!FileSelectTextYDisp*$06)
	dw !FileSelectTextInitialYDisp+(!FileSelectTextYDisp*$07),!FileSelectTextInitialYDisp+$10+(!FileSelectTextYDisp*$07)
	dw !FileSelectTextInitialYDisp+(!FileSelectTextYDisp*$08),!FileSelectTextInitialYDisp+$10+(!FileSelectTextYDisp*$08)
..End:

.FileSelectLineRoutines:
	dw .NotOnSelection,.File1
	dw .NotOnSelection,.File2
	dw .NotOnSelection,.File3
	dw .NotOnSelection,.EraseFileMenu
	dw .NotOnSelection,.HighScores
	dw .NotOnSelection,.MusicOption
	dw .NotOnSelection,.SoundOption
	dw .NotOnSelection,.SongTest
	dw .NotOnSelection,.NotOnSelection

.EraseFileLineRoutines:
	dw .NotOnSelection,.EraseFile1
	dw .NotOnSelection,.EraseFile2
	dw .NotOnSelection,.EraseFile3
	dw .NotOnSelection,.FileSelectMenu
	dw .NotOnSelection,.EraseHighScores
	dw .NotOnSelection,.NotOnSelection
	dw .NotOnSelection,.NotOnSelection
	dw .NotOnSelection,.NotOnSelection
	dw .NotOnSelection,.NotOnSelection

.FileSelectOptionIndex:
	dw $00FF,$0000
	dw $00FF,$0002
	dw $00FF,$0004
	dw $00FF,$0006
	dw $00FF,$0008
	dw $00FF,$000A
	dw $00FF,$000C
	dw $00FF,$000E
	dw $00FF,$00FF

.EraseFileOptionIndex:
	dw $00FF,$0010
	dw $00FF,$0012
	dw $00FF,$0014
	dw $00FF,$0016
	dw $00FF,$0018
	dw $00FF,$00FF
	dw $00FF,$00FF
	dw $00FF,$00FF
	dw $00FF,$00FF

.NotOnSelection:
	JML.l $81D8F5

.File1:
	LDA.w #$0000
	BRA.b +

.File2:
	LDA.w #$0001
	BRA.b +

.File3:
	LDA.w #$0002
+:
	STA.l !CurrentSaveFile
	JML.l $81D8F6

.HighScores:
	JML.l $81D91D

.MusicOption:
	JML.l $81D944

.SoundOption:
	JML.l $81D96E

.EraseFile1:
	LDA.w #$0000
	BRA.b +

.EraseFile2:
	LDA.w #$0001
	BRA.b +

.EraseFile3:
	LDA.w #$0002
+:
	STA.l !CurrentSaveFile
	JSR.w GetSRAMPointer
	LDA.w $0AF1
	AND.w #$E0C0
	BEQ.b +
	JSR.w ReadSRAMByte
	INC
	JSR.w StoreSRAMByte
	BRA.b ++

.EraseHighScores:
	LDA.w $0AF1
	AND.w #$E0C0
	BEQ.b +
	LDA.l !HighScoreChecksum
	INC
	STA.l !HighScoreChecksum
++:
	JSL.l VerifySaveFiles
	LDA.w #$0218				;\ Play "Exploding Dinosaur" sound 
	LDY.w #$F00A				;|
	JSL.l $839D6A				;/
+:
	JML.l $81D8F5

.FileSelectMenu:
	LDA.w $0AF1
	AND.w #$E0C0
	BEQ.b ++++
	LDA.w #$0001
	STA.w $044D
	BRA.b +++

.EraseFileMenu:
	LDA.w $0AF1
	AND.w #$E0C0
	BEQ.b ++++
	LDA.w #$0003
	STA.w $044D
	BRA.b +++

.IncrementSong:
	LDA.l !SongTestSetting
	INC
	CMP.w #((.MusicPtrs_End-.MusicPtrs)/$04)
	BCC.b +
	LDA.w #$0000
+:
	BRA.b ++

.DecrementSong:
	LDA.l !SongTestSetting
	DEC
	BPL.b ++
	LDA.w #((.MusicPtrs_End-.MusicPtrs)/$04)-$01
++:
	STA.l !SongTestSetting
+++:
	LDA.w #$2118				;\ Play "Open Message Box" sound 
	LDY.w #$F00A				;|
	JSL.l $839D6A				;/
++++:
	JML.l $81D8F5

.SongTest:
	LDA.w $0AF1
	AND.w #$E0C0
	BEQ.b +
	BIT.w #$4040				; A/B Button
	BNE.b .DecrementSong
	BIT.w #$8080				; X/Y Button
	BNE.b .IncrementSong
	LDA.l !SongTestSetting			; Select button
	ASL
	ASL
	TAX
	LDA.l .MusicPtrs,x
	PHA
	LDA.l .MusicPtrs+$02,x
	TAY
	PLX
	JSL.l $839C8D
+:
	JML.l $81D8F5

.MusicPtrs:
	dd $BF8000				; Title Screen
	dd $BE8000				; Intro
	dd $B78000				; Raptor Rap
	dd $B68000				; High score theme
	dd $B48000				; T-Rex Trample
	dd $B88000				; Triceratops Trot
	dd $B98000				; Submarine Symphony
	dd $BA8000				; Gallamimus Gallop
	dd $BD8000				; Interior Interlude
	dd $BC8000				; Elevator Music 1
	dd $BB8000				; Elevator Music 2
..End:

LoadSavedHighScoreData:
	LDX.w #$007C
	TXY
-
	LDA.l !HighScoreData,x
	STA.w $2000,y
	DEX
	DEX
	DEY
	DEY
	BPL.b -
	RTL

SaveNewHighScore:
	JSL.l SaveHighScores
	JML.l $819EF1

SaveHighScores:
	LDA.w #$0000
	PHA
	LDY.w #$007C
	TYX
-
	LDA.w $2000,y
	STA.l !HighScoreData,x
	CLC
	ADC.b !CurrentScoreChecksumRAM,s
	STA.b !CurrentScoreChecksumRAM,s
	DEX
	DEX
	DEY
	DEY
	BPL.b -
	SEC
	SBC.w #!ChecksumCompliment
	STA.l !HighScoreChecksum
	PLA
	RTL

SaveMusicSetting:
	EOR.w #$00FF
	STA.w $02EF
	STA.l !MusicSetting
	RTL

SaveSoundSetting:
	JSL.l $839D6A
	LDA.w $02F1
	STA.l !SoundSetting
	RTL

JPIconHijack:
	LDA.w $044D
	BEQ.b +
	JML.l $81DAEB

+:
	SEP.b #$20
	LDA.b #$11
	JML.l $81DB47

HandleNewTitleScreenFiles:
	LDX.w #$8FC4C2					;\ Load Jurassic Park font file rather than the open menu graphics on the title screen.
	LDA.w #$8FC4C2>>16				;/
	LDY.w #$7FE400
	JSL.l $81B18B
-:
	LDA.w $0467
	BNE.b -
	LDA.w #$2118					;\ Load buffered Jurassic Park Font graphics 
	STA.w $045F					;|
	LDA.w #$6300					;|
	STA.w $0461					;|
	LDA.w #$7FE400					;|
	STA.w $0463					;|
	LDA.w #$7FE400>>16				;|
	STA.w $0465					;|
	LDA.w #$1800					;|
	STA.w $0467					;/
	LDX.w #$8CEE74					;\ Load title screen palette
	LDA.w #$8CEE74>>16				;|
	JSL.l $81B3A6					;/
	PHB						;\ Overwrite menu palette with a new palette
	REP.b #$30					;|
	LDA.w #$0020					;|
	LDX.w #JurassicParkFontPalette			;|
	LDY.w #$7FE1E0					;|
	MVN $7FE1E0>>16,JurassicParkFontPalette>>16	;/
	LDA.w #CursorGFX_End-CursorGFX			;\ Buffer fixed cursor graphics that will handle the new palette
	LDX.w #CursorGFX				;|
	LDY.w #$7F8180					;|
	MVN $7F8180>>16,CursorGFX>>16			;|
	PLB						;/
-:							;\ Load buffered cursor graphics 
	LDA.w $0467					;|
	BNE.b -						;|
	LDA.w #$2118					;|
	STA.w $045F					;|
	LDA.w #$7000					;|
	STA.w $0461					;|
	LDA.w #$7F8000					;|
	STA.w $0463					;|
	LDA.w #$7F8000>>16				;|
	STA.w $0465					;|
	LDA.w #$0400					;|
	STA.w $0467					;/
	RTL

JurassicParkFontPalette:
	dw $0000,$007E,$01DA,$1EBF,$029E,$031E,$037E,$03DE,$2FDE,$0048,$02DD,$0259,$01D5,$0151,$00ED,$5FDE

StoreSRAMByteAndStoreTempChecksum:
	STA.b [$F8],y
	CLC
	ADC.b !CurrentFileChecksumRAM+$02,s
	STA.b !CurrentFileChecksumRAM+$02,s
	BRA.b +

ReadSRAMByte:
	LDA.b [$F8],y
	BRA.b +

ORASRAMByte:
	ORA.b [$F8],y
	BRA.b +

StoreSRAMByte:
	STA.b [$F8],y
+:
	INY
	INY
	CPY.w #$8000
	BCC.b +
	LDY.w #$0000
	INC.b $FA
+:
	RTS

GetSRAMPointer:
	ASL
	ASL
	TAX
	LDA.l SaveFileLocations,x
	STA.b $FC
	STZ.b $F8
	TAY
	INY
	INY
	CPY.w #$8000
	BCC.b +
	LDY.w #$0000
	INC.b $FA
+:
	LDA.l SaveFileLocations+$02,x
	STA.b $FA
	STA.b $FE
	RTS

